'////////////////////////////////////////////////////////////////////////////
' $Workfile: ADSIExplore.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/ADSIExplore.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Enumerate all ADSI objects on the specified computer

' Explicit variable declaration and standard globals
Option Explicit
Dim g_sScriptPath, g_sScriptName, g_sScriptFolder, g_sVersion
Dim g_nTraceLevel
Dim g_oShell, g_oFSO
Dim s, ix, i

' Set standard globals and create global objects
g_sVersion = "1.0"
g_sScriptPath = Wscript.ScriptFullName
g_sScriptName = Wscript.ScriptName
g_sScriptFolder = Left(g_sScriptPath, Len(g_sScriptPath) - Len(g_sScriptName))
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")

' Setup trace control from WSHTRACE environment variable
i = g_oShell.Environment("Process").Item("WSHTRACE")
If IsNumeric(i) Then g_nTraceLevel = CInt(i) Else g_nTraceLevel = 0

' Check for -help, -? etc help request on command line
If Wscript.Arguments.Count > 0 Then
    s = LCase(Wscript.Arguments(0))
    If (s = "-help") Or (s = "-?") Or (s = "/help") Or (s = "/?") Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
End If

' Globals used by Main etc
Dim g_bSubObjSwitch, g_bVerboseSwitch, g_aFilters

' Show signon banner, then call Main function
ix = Instr(g_sScriptName, ".")
If ix <> 0 Then s = Left(g_sScriptName, ix - 1) Else s = g_sScriptName
Wscript.Echo s & " version " & g_sVersion & vbCRLF
i = Main

' Release standard global objects, then exit script
Set g_oFSO = Nothing
Set g_oShell = Nothing
Wscript.Quit(i)

'////////////////////////////////////////////////////////////////////////////
' Main
' Main function called to embody primary script logic
'
' Returns           Exit code from script
'
Function Main
    Trace 1, "+++Main"
    Dim sAdsPath, oAdsObj, nCount, oArgs, oArg

    ' Parse command line into globals
    Set oArgs = CreateObject("MTP.CmdArgs")
    oArgs.FromCollection(Wscript.Arguments)     ' Get command line
    If oArgs.TypedCount(1+4) < 1 Then           ' If missing path..
        ShowHelpMessage
        Wscript.Quit(0)
    End If
    sAdsPath = oArgs.TypedItem(1+4, 0).Text     ' Get ADSI path name
    g_bSubObjSwitch = oArgs.TestSwitch("c") Or oArgs.TestSwitch("s")
    g_bVerboseSwitch = oArgs.TestSwitch("v")
    ReDim g_aFilters(100) : ix = 0 : nCount = 0
    Do
        Set oArg = oArgs.FindArg(2, "f", ix)    ' Find /f switch
        If Not oArg Is Nothing Then
            ix = oArg.Index + 1                 ' Ready for next
            g_aFilters(nCount) = oArg.Value     ' Get filter value
            nCount = nCount + 1
        End if
    Loop Until oArg Is Nothing
    If nCount > 0 Then
        ReDim Preserve g_aFilters(nCount - 1)   ' Trim array
    Else
        g_aFilters = Empty                      ' Delete array
    End If
    Set oArgs = Nothing

    ' Prepare the ADSI object
    On Error Resume Next
    Set oAdsObj = GetObject(sAdsPath)
    If Err.Number <> 0 Then
        Wscript.Echo sAdsPath & ": not found (0x" & Hex(Err.Number) & ")"
        Wscript.Quit(Err.Number)
    End If
    On Error Goto 0

    ' Display heading, then dump primary object and contained
    Wscript.StdOut.WriteLine "ADSI Dump of: " & sAdsPath
    Wscript.StdOut.WriteLine
    If g_bVerboseSwitch Then
        DumpSchema "", oAdsObj
        Wscript.StdOut.WriteLine
    End If
    DumpAll "", oAdsObj

    ' Return value is passed to Wscript.Quit as script exit code
    Main = 0
End Function

'////////////////////////////////////////////////////////////////////////////
' DumpSchema
' Dump the schema for the specified ADSI object
'
' sPrefix           Spaces prefix for indenting
' oAdsObj           Object to dump
' 
Sub DumpSchema(sPrefix, oAdsObj)
    Trace 1, "+++DumpSchema"
    Dim sData, oClass, sList

    ' Get the class object for the ADSI object
    Set oClass = GetObject(oAdsObj.Schema)

    ' Dump the class contents (objects)
    Wscript.StdOut.WriteLine sPrefix & "Schema for class: " & oClass.Name
    If oClass.Container Then
        Wscript.StdOut.WriteLine sPrefix & "    Contained objects:"
        sList = ""
        For Each sData In oClass.Containment
            If sList = "" Then
                sList = sData
            Else
                sList = sList & ", " & sData
            End If
        Next
        Wscript.StdOut.WriteLine sPrefix & "        " & sList
    End If

    ' Dump the class properties
    Wscript.StdOut.WriteLine sPrefix & "    Properties (O=optional, M=mandatory):"
    For Each sData In oClass.MandatoryProperties
        Wscript.StdOut.WriteLine sPrefix & "        (M) " & sData
    Next
    For Each sData in oClass.OptionalProperties
        Wscript.StdOut.WriteLine sPrefix & "        (O) " & sData
    Next
End Sub

'////////////////////////////////////////////////////////////////////////////
' DumpAll
' Dump object and any objects it contains, recursively
'
' sPrefix           Spaces prefix for indenting
' oAdsObj           Object to dump
'
Sub DumpAll(sPrefix, oAdsObj)
    Trace 2, "+++DumpAll"
    Dim oSubObj

    ' Dump the object itself
    DumpObject sPrefix, oAdsObj
    Wscript.StdOut.WriteLine

    ' If object is a container, enumerate all contained objects
    If g_bSubObjSwitch And GetObject(oAdsObj.Schema).Container = True Then
        If Not IsEmpty(g_aFilters) Then
            oAdsObj.Filter = g_aFilters         ' Setup filters
        End If
        For Each oSubObj In oAdsObj
            DumpAll sPrefix & "    ", oSubObj
        Next
    End If
End Sub

'////////////////////////////////////////////////////////////////////////////
' DumpObject
' Dump individual ADSI object (also dumps related object info)
'
' sPrefix           Spaces prefix for indenting
' oAdsObj           Object to dump
'
Sub DumpObject(sPrefix, oAdsObj)
    Trace 3, "+++DumpObject"
    Dim nPropCount, ix, sName, vData, sType, oPropEntry

    ' Show basic object information
    Wscript.StdOut.WriteLine sPrefix & "*** " & oAdsObj.Class & ": " & oAdsObj.Name
    Wscript.StdOut.WriteLine sPrefix & "AdsPath: " & oAdsObj.AdsPath & "," & oAdsObj.Class

    ' Load the property cache and get property count
    oAdsObj.GetInfo
    On Error Resume Next
    nPropCount = oAdsObj.PropertyCount
    If Err.Number <> 0 Then nPropCount = 0
    On Error Goto 0

    ' Process any class-specific information
    Select Case oAdsObj.Class
        Case "Group", "LocalGroup"
            Wscript.StdOut.WriteLine sPrefix & "Members:"
            Wscript.StdOut.WriteLine sPrefix & "    " & GetList(oAdsObj.Members)
        Case "User"
            Wscript.StdOut.WriteLine sPrefix & "Groups:"
            Wscript.StdOut.WriteLine sPrefix & "    " & GetList(oAdsObj.Groups)
        Case "Service"
            Wscript.StdOut.WriteLine sPrefix & "Status=" & oAdsObj.Status
        Case "FileService"
            Wscript.StdOut.WriteLine sPrefix & "Sessions:"
            Wscript.StdOut.WriteLine sPrefix & "    " & GetList(oAdsObj.Sessions)
    End Select

    ' Dump the property list (if verbose)
    If nPropCount <> 0 And g_bVerboseSwitch Then
        Wscript.StdOut.WriteLine sPrefix & "Properties (" & nPropCount & "):"
        For ix = 0 To nPropCount - 1
            Set oPropEntry = oAdsObj.Item(ix)
            If oPropEntry.AdsType <> 0 Then
                vData = oAdsObj.Get(oPropEntry.Name)
                sType = oPropEntry.AdsType & "," & TypeName(vData)
                sName = oPropEntry.Name & "[" & sType & "]="
                Wscript.StdOut.WriteLine sPrefix & "    " & sName & CStr(vData)
            End If
        Next
    End If
End Sub

' Build comma delimited name list
Function GetList(oColl)
    Dim oObj

    GetList = ""
    For Each oObj In oColl                      ' For each object
        If GetList = "" Then                    ' if first item..
            GetList = oObj.Name
        Else
            GetList = GetList & ", " & oObj.Name
        End If
    Next
End Function

'////////////////////////////////////////////////////////////////////////////
' ShowHelpMessage
' Display help message
'
Sub ShowHelpMessage
    Trace 1, "+++ShowHelpMessage"
    Wscript.Echo "usage: adsiexplore <ADSI-path> [<switches>]"
    Wscript.Echo "    /c      Dump contained (child) objects"
    Wscript.Echo "    /s      Same as /c"
    Wscript.Echo "    /f:cls  Filter for specified class (def: all)"
    Wscript.Echo "            (Multiple /f switches are allowed)"
    Wscript.Echo "    /v      Verbose output (schema and other details)"
    Wscript.Echo ""           
    Wscript.Echo "Enumerates all ADSI properties in the WinNT space for"
    Wscript.Echo "the specified ADSI path (e.g. WinNT://computer,computer)."
End Sub

'////////////////////////////////////////////////////////////////////////////
' Trace
' Debug trace output controlled by g_nTraceLevel and WSHTRACE env var
'
' nLevel            Trace level. Only display if >= WSHTRACE env var
' sText             Text to display
'
Sub Trace(nLevel, sText)
    if g_nTraceLevel > nLevel Then Wscript.Echo sText
End Sub

'////////////////////////////////////////////////////////////////////////////
 