'////////////////////////////////////////////////////////////////////////////
' $Workfile: Animal.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/Animal.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Plays the ancient Animal computer game

Option Explicit
Dim oAnimals, sDatabase, n

' Form data file path name
sDatabase = Wscript.ScriptFullName
sDatabase = Left(sDatabase, InStrRev(sDatabase, ".") - 1) & ".dat"

' Load database and play game, then save if database is dirty
Set oAnimals = New AnimalSet
oAnimals.Reset
oAnimals.Load sDatabase
If PlayGame(oAnimals) Then
    oAnimals.Save sDatabase
End If
Set oAnimals = Nothing
Wscript.Quit(0)

'////////////////////////////////////////////////////////////////////////////
' PlayGame
' Plays the actual animal game
'
' oAnimals          Ref to AnimalSet object ready to play the game
' Returns           True if the set has changed (new animals)
'
Function PlayGame(ByRef oAnimals)
    Dim sMsg, nComputer, nHuman
    nComputer = 0 : nHuman = 0
    sMsg = ""
    PlayGame = False                            ' Not dirty yet
    Dim oCursor, n, sAnimal, sQuestion, bYesForNew
    Do While True
        Set oCursor = oAnimals.GetCursor        ' Get the game cursor
        n = MsgBox(sMsg & "Computer: " & nComputer & vbCRLF & "Human: " & nHuman & vbCRLF & "Think of an animal, then click OK to play or Cancel to end the game.", vbOKCancel, "Animal")
        If n = vbCancel Then Exit Do
        Do While oCursor.IsQuestion
            n = MsgBox(oCursor.Text, vbYesNo, "Animal")
            If n = vbYes Then
                oCursor.FollowYes
            Else
                oCursor.FollowNo
            End If
        Loop
        n = MsgBox("Is it a " & oCursor.Text & "?", vbYesNo, "Animal")
        If n = vbYes Then
            sMsg = "I won!! Let's play again." & vbCRLF
            nComputer = nComputer + 1
        Else
            sAnimal = InputBox("You won!! What animal were you thinking of?", "Animal")
            sAnimal = Trim(sAnimal)
            If sAnimal = "" Then Wscript.Quit(1)
            Do
                sQuestion = InputBox("Please type a yes/no question to distinguish a " & sAnimal & " from a " & oCursor.Text & ".", "Animal")
                sQuestion = Trim(sQuestion)
                If Instr(sQuestion, "?") = 0 Then
                    n = MsgBox("Please type a question (with a ""?"" at the end)!", vbOK, "Animal")
                    sQuestion = ""
                End If
            Loop While sQuestion = ""
            n = MsgBox("The question is: """ & sQuestion & """" & vbCRLF & "For a " & sAnimal & " the answer would be?", vbYesNo, "Animal")
            If n = vbYes Then
                oCursor.AddNewAnimal sAnimal, sQuestion, True
            Else
                oCursor.AddNewAnimal sAnimal, sQuestion, False
            End If
            PlayGame = True
            sMsg = "I want revenge!" & vbCRLF
            nHuman = nHuman + 1
        End If
        Set oCursor = Nothing                   ' Release the cursor
    Loop
End Function

'////////////////////////////////////////////////////////////////////////////
' IsValidObject
' Checks to see if a variable is a non-nothing object
'
' oObj              Object reference to check
' Returns           True if the var contains a valid object
'
Function IsValidObject(ByRef oObj)
    IsValidObject = False
    If IsObject(oObj) Then
        If Not oObj Is Nothing Then
            IsValidObject = True
        End If
    End If
End Function

'////////////////////////////////////////////////////////////////////////////
' AnimalCursor class
' Manages an animal game by traversing the animal tree
'
Class AnimalCursor
    Private m_oTree                             ' Tree we are attached to
    Private m_oNode                             ' Current node in tree
    
    ' Attach/detach to/from a tree
    Public Sub Attach(ByRef oTree, ByRef oNode) ' Attach cursor to tree
        Set m_oTree = oTree
        Set m_oNode = oNode
    End Sub
    
    Public Sub Detach                           ' Detach cursor from tree
        Set m_oTree = Nothing
        Set m_oNode = Nothing
    End Sub
    
    ' IsAnimal and IsQuestion tests
    Public Property Get IsAnimal
        If IsValidObject(m_oNode) Then
            If IsValidObject(m_oNode.Yes) Then
                IsAnimal = False                ' Animals are leaves
            Else
                IsAnimal = True
            End If
        Else
            IsAnimal = False
        End If
    End Property
    
    Public Property Get IsQuestion
        If IsValidObject(m_oNode) Then
            IsQuestion = Not IsAnimal           ' Questions are interior nodes
        Else
            IsQuestion = False
        End If
    End Property

    ' Traverse to next node
    Public Sub FollowYes
        If IsValidObject(m_oNode.Yes) Then      ' Only if valid
            Set m_oNode = m_oNode.Yes
        End If
    End Sub
    
    Public Sub FollowNo
        If IsValidObject(m_oNode.No) Then       ' Only if valid
            Set m_oNode = m_oNode.No
        End If
    End Sub

    Public Property Get Text
        Text = m_oNode.Text                     ' Current node text
    End Property

    ' Add new animal to tree (at end of game)
    Public Sub AddNewAnimal(sAnimal, sQuestion, bYesForNew)
        Dim oYesNode, oNoNode
        If IsValidObject(m_oNode) Then          ' Only if valid cursor
            Set oYesNode = m_oTree.NewNode(sAnimal, Nothing, Nothing)
            Set oNoNode = m_oTree.NewNode(m_oNode.Text, Nothing, Nothing)
            m_oNode.Text = sQuestion            ' New interior node
            If bYesForNew Then                  ' Link in correct way
                Set m_oNode.Yes = oYesNode
                Set m_oNode.No = oNoNode
            Else
                Set m_oNode.Yes = oNoNode
                Set m_oNode.No = oYesNode
            End If
        End If
    End Sub
End Class

'////////////////////////////////////////////////////////////////////////////
' AnimalNode class
' Stores data for a single node in the animal tree
'
Class AnimalNode
    Public Text                                 ' Current node text
    Public Yes                                  ' Yes answer node ref
    Public No                                   ' No answer node ref
    Public Index                                ' Used when saving
End Class

'////////////////////////////////////////////////////////////////////////////
' AnimalSet class
' Stores the complete set of animal nodes (leaves are animals)
'
Class AnimalSet
    Private m_oRoot                             ' Animal tree root
    Private m_nIndex                            ' Used when numbering nodes
    
    ' Reset the tree (delete all nodes)
    Public Sub Reset
        DeleteNode m_oRoot                      ' Delete entire tree
    End Sub
    
    Private Sub DeleteNode(ByRef oNode)
        If IsValidObject(oNode) Then            ' If valid node..
            DeleteNode oNode.Yes                ' Delete Yes sub-tree
            DeleteNode oNode.No                 ' Delete No sub-tree
        End If
        Set oNode = Nothing                     ' Delete the node itself
    End Sub
    
    ' Load the tree from a file
    Public Sub Load(sFilename)
        Dim oFSO, oFile, nLastNode, nNodeSize
        Dim sLine, vData, oNode, ix, oNodes()
        Reset
        Set oFSO = CreateObject("Scripting.FileSystemObject")
        If Not oFSO.FileExists(sFilename) Then
            LoadBuiltin                         ' Load the builtin DB
            Set oFSO = Nothing
            Exit Sub                            ' .. and done
        End If
        Set oFile = oFSO.OpenTextFile(sFilename, 1)
        nLastNode = 0 : nNodeSize = 0
        Do While Not oFile.AtEndOfStream
            sLine = Trim(oFile.ReadLine)
            If Len(sLine) > 0 And Left(sLine, 1) <> ";" Then
                vData = Split(sLine, ",", 3)
                Set oNode = NewNode(vData(2), vData(0), vData(1))
                nLastNode = nLastNode + 1
                If nLastNode > nNodeSize Then
                    nNodeSize = nLastNode + 100
                    ReDim Preserve oNodes(nNodeSize)
                End If
                Set oNodes(nLastNode) = oNode
                Set oNode = Nothing
            End If
        Loop
        Set oFile = Nothing
        For ix = 1 To nLastNode
            n = oNodes(ix).Yes
            If n <> 0 Then
                Set oNodes(ix).Yes = oNodes(n)
            Else
                Set oNodes(ix).Yes = Nothing
            End If
            n = oNodes(ix).No
            If n <> 0 Then
                Set oNodes(ix).No = oNodes(n)
            Else
                Set oNodes(ix).No = Nothing
            End If
        Next
        Set m_oRoot = oNodes(1)                 ' Set root node
        Erase oNodes
    End Sub
    
    ' Load the builtin animal database
    Public Sub LoadBuiltin
        Reset
        Set m_oRoot = NewNode("Does it live in the sea?", Nothing, Nothing)
        Set m_oRoot.No = NewNode("dog", Nothing, Nothing)
        Set m_oRoot.Yes = NewNode("fish", Nothing, Nothing)
    End Sub
    
    ' Save the tree to a file
    Public Sub Save(sFilename)
        Dim oFSO, oFile
        m_nIndex = 1                            ' Start index
        NumberNodes m_oRoot                     ' Number all nodes
        Set oFSO = CreateObject("Scripting.FileSystemObject")
        Set oFile = oFSO.CreateTextFile(sFilename, True)
        oFile.WriteLine ";ANIMAL data (" & m_nIndex - 1 & ")"
        SaveTree oFile, m_oRoot
        oFile.Close
        Set oFile = Nothing
        Set oFSO = Nothing
    End Sub

    Private Sub SaveTree(ByRef oFile, ByRef oNode)
        Dim nYes, nNo
        If IsValidObject(oNode) Then
            If IsValidObject(oNode.Yes) Then
                nYes = oNode.Yes.Index
            Else
                nYes = 0
            End If
            If IsValidObject(oNode.No) Then
                nNo = oNode.No.Index
            Else
                nNo = 0
            End If
            oFile.WriteLine nYes & "," & nNo & "," & oNode.Text
            SaveTree oFile, oNode.Yes
            SaveTree oFile, oNode.No
        End If
    End Sub
    
    Private Sub NumberNodes(ByRef oNode)
        If IsValidObject(oNode) Then
            oNode.Index = m_nIndex              ' Number this node
            m_nIndex = m_nIndex + 1             ' Bump the index
            NumberNodes oNode.Yes               ' Number yes tree
            NumberNodes oNode.No                ' .. and no tree
        End If
    End Sub

    ' Get a cursor
    Public Function GetCursor
        Set GetCursor = New AnimalCursor        ' Create a new cursor
        GetCursor.Attach Me, m_oRoot            ' Attach to this tree
    End Function
    
    ' Create a new node
    Public Function NewNode(sText, ByRef oYes, ByRef oNo)
        Set NewNode = New AnimalNode            ' Create new node
        NewNode.Text = sText
        If IsObject(oYes) Then
            Set NewNode.Yes = oYes
            Set NewNode.No = oNo
        Else
            NewNode.Yes = oYes
            NewNode.No = oNo
        End If
    End Function
End Class

'////////////////////////////////////////////////////////////////////////////
