'////////////////////////////////////////////////////////////////////////////
' $Workfile: DumpUsers.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/DumpUsers.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Dump user accounts to an Excel spreadsheet

' Explicit variable declaration and standard globals
Option Explicit
Dim g_sScriptPath, g_sScriptName, g_sScriptFolder, g_sVersion
Dim g_nTraceLevel
Dim g_oShell, g_oFSO
Dim s, ix, i

' Set standard globals and create global objects
g_sVersion = "1.0"
g_sScriptPath = Wscript.ScriptFullName
g_sScriptName = Wscript.ScriptName
g_sScriptFolder = Left(g_sScriptPath, Len(g_sScriptPath) - Len(g_sScriptName))
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")

' Setup trace control from WSHTRACE environment variable
i = g_oShell.Environment("Process").Item("WSHTRACE")
If IsNumeric(i) Then g_nTraceLevel = CInt(i) Else g_nTraceLevel = 0

' Check for -help, -? etc help request on command line
If Wscript.Arguments.Count > 0 Then
    s = LCase(Wscript.Arguments(0))
    If (s = "-help") Or (s = "-?") Or (s = "/help") Or (s = "/?") Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
End If

' Show signon banner, then call Main function
ix = Instr(g_sScriptName, ".")
If ix <> 0 Then s = Left(g_sScriptName, ix - 1) Else s = g_sScriptName
Wscript.Echo s & " version " & g_sVersion & vbCRLF
i = Main

' Release standard global objects, then exit script
Set g_oFSO = Nothing
Set g_oShell = Nothing
Wscript.Quit(i)

'////////////////////////////////////////////////////////////////////////////
' Main
' Main function called to embody primary script logic
'
' Returns           Exit code from script
'
Function Main
    Trace 1, "+++Main"
    Dim sAdsPath, sExcelPath, oSheet, oExcel, oUser, oAdsObj

    ' Validate command line and get args
    If Wscript.Arguments.Count < 2 Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
    sAdsPath =  "WinNT://" & Wscript.Arguments(0)
    sExcelPath = g_oFSO.GetAbsolutePathName(Wscript.Arguments(1))

    ' Prepare ADSI computer/domain object
    On Error Resume Next
    Set oAdsObj = GetObject(sAdsPath)
    If Err.Number <> 0 Then
        Wscript.Echo sAdsPath & ": not found (0x" & Hex(Err.Number) & ")"
        Wscript.Quit(Err.Number)
    End If
    On Error Goto 0

    ' Prepare spreadsheet
    Set oExcel = CreateObject("Excel.Application")
    oExcel.Workbooks.Add
    oExcel.ActiveWorkbook.Worksheets.Add
    Set oSheet = oExcel.ActiveWorkbook.Worksheets(1)
    oSheet.Cells.Font.Size = 8
    oSheet.Name = "User Dump"
    oSheet.Cells(1,1).Value = "Dump of User Accounts on: " & Wscript.Arguments(0)
    oSheet.Cells(1,1).Font.Bold = True
    oSheet.Cells(1,1).Font.Size = 10
    oSheet.Range("A3:I3").Font.Bold = True
    oSheet.Range("A3:I3").Interior.Color = RGB(192,192,192)
    SetupCol oSheet, 3, 1,  12, "Name"
    SetupCol oSheet, 3, 2,  18, "Full Name"
    SetupCol oSheet, 3, 3,  10, "Home Drive"
    SetupCol oSheet, 3, 4,  12, "Home Dir"
    SetupCol oSheet, 3, 5,  12, "Login Script"
    SetupCol oSheet, 3, 6,  8, "User Flags"
    SetupCol oSheet, 3, 7,  12, "Profile"
    SetupCol oSheet, 3, 8,  36, "Description"
    SetupCol oSheet, 3, 9,  36, "Groups"

    ' Enumerate all users in the computer/domain
    oAdsObj.Filter = Array("User")              ' Filter user accounts only
    ix = 0
    For Each oUser In oAdsObj                   ' For each user account..
        DumpAccount oSheet, ix, oUser           ' Go add to sheet
        ix = ix + 1                             ' Bump index
    Next

    ' Save spreadsheet and close
    oExcel.ActiveWorkbook.SaveAs sExcelPath
    oExcel.ActiveWorkbook.Close
    Set oSheet = Nothing
    Set oExcel = Nothing
    Set oAdsObj = Nothing
    
    ' Return value is passed to Wscript.Quit as script exit code
    Main = 0
End Function

Sub SetupCol(oSheet, nRow, nCol, nWidth, sTitle)
    oSheet.Cells(nRow, nCol).Value = sTitle
    oSheet.Cells(nRow, nCol).ColumnWidth = nWidth
End Sub

'////////////////////////////////////////////////////////////////////////////
' DumpAccount
' Dump user accounts
'
' oSheet            Spreadsheet object
' ix                Account index
' oUser             User account object
'
Sub DumpAccount(oSheet, ix, oUser)
    Dim oObj, sList

    ' Build group list
    sList = ""
    For Each oObj In oUser.Groups
        If sList = "" Then
            sList = oObj.Name
        Else
            sList = sList & ", " & oObj.Name
        End If
    Next

    ' Setup cell values
    oSheet.Cells(4 + ix,  1).Value = oUser.Name
    oSheet.Cells(4 + ix,  2).Value = oUser.FullName
    oSheet.Cells(4 + ix,  3).Value = oUser.HomeDirDrive
    oSheet.Cells(4 + ix,  4).Value = oUser.HomeDirectory
    oSheet.Cells(4 + ix,  5).Value = oUser.LoginScript
    oSheet.Cells(4 + ix,  6).Value = "0x" & Hex(oUser.UserFlags)
    oSheet.Cells(4 + ix,  7).Value = oUser.Profile
    oSheet.Cells(4 + ix,  8).Value = oUser.Description
    oSheet.Cells(4 + ix,  9).Value = sList
End Sub

'////////////////////////////////////////////////////////////////////////////
' ShowHelpMessage
' Display help message
'
Sub ShowHelpMessage
    Trace 1, "+++ShowHelpMessage"
    Wscript.Echo "usage: dumpusers <computer|domain> <xls-file>"
    Wscript.Echo
    Wscript.Echo "Dumps user account information into the specified Excel"
    Wscript.Echo "spreadsheet file. Specify either a computer name or a"
    Wscript.Echo "domain name. To improve performance, append "",computer"""
    Wscript.Echo "to computer names and "",domain"" to domains name."
    Wscript.Echo "For example:"
    Wscript.Echo
    Wscript.Echo "    dumpusers mydomain,domain mydomain.xls"
    Wscript.Echo
    Wscript.Echo "This will dump users in the domain MYDOMAIN to MYDOMAIN.XLS"
End Sub

'////////////////////////////////////////////////////////////////////////////
' Trace
' Debug trace output controlled by g_nTraceLevel and WSHTRACE env var
'
' nLevel            Trace level. Only display if >= WSHTRACE env var
' sText             Text to display
'
Sub Trace(nLevel, sText)
    if g_nTraceLevel > nLevel Then Wscript.Echo sText
End Sub

'////////////////////////////////////////////////////////////////////////////
 