'////////////////////////////////////////////////////////////////////////////
' $Workfile: FindMovedFiles.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/FindMovedFiles.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Find all files moved in a tree

' Explicit variable declaration and standard globals
Option Explicit
Dim g_sScriptPath, g_sScriptName, g_sScriptFolder, g_sVersion
Dim g_nTraceLevel
Dim g_oShell, g_oFSO
Dim s, ix, i

' Set standard globals and create global objects
g_sVersion = "1.0"
g_sScriptPath = Wscript.ScriptFullName
g_sScriptName = Wscript.ScriptName
g_sScriptFolder = Left(g_sScriptPath, Len(g_sScriptPath) - Len(g_sScriptName))
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")

' Setup trace control from WSHTRACE environment variable
i = g_oShell.Environment("Process").Item("WSHTRACE")
If IsNumeric(i) Then g_nTraceLevel = CInt(i) Else g_nTraceLevel = 0

' Check for -help, -? etc help request on command line
If Wscript.Arguments.Count > 0 Then
    s = LCase(Wscript.Arguments(0))
    If (s = "-help") Or (s = "-?") Or (s = "/help") Or (s = "/?") Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
End If

' Show signon banner, then call Main function
ix = Instr(g_sScriptName, ".")
If ix <> 0 Then s = Left(g_sScriptName, ix - 1) Else s = g_sScriptName
Wscript.Echo s & " version " & g_sVersion & vbCRLF
i = Main

' Release standard global objects, then exit script
Set g_oFSO = Nothing
Set g_oShell = Nothing
Wscript.Quit(i)

'////////////////////////////////////////////////////////////////////////////
' Main
' Main function called to embody primary script logic
'
' Returns           Exit code from script
'
Function Main
    Trace 1, "+++Main"
    Dim sSrcPath, sDstPath, oSrcDict, oDstDict, oSrcRoot, oDstRoot, nCount

    ' Validate arguments (must be at least two, and both must be valid dirs)
    If Wscript.Arguments.Count < 2 Then ShowHelpMessage : Wscript.Quit(0)
    sSrcPath = g_oFSO.BuildPath(Wscript.Arguments(0), ".")
    If Not g_oFSO.FolderExists(sSrcPath) Then
        Wscript.Echo Wscript.Arguments(0) & " does not exist"
        Wscript.Quit(0)
    End If
    sDstPath = g_oFSO.BuildPath(Wscript.Arguments(1), ".")
    If Not g_oFSO.FolderExists(sDstPath) Then
        Wscript.Echo Wscript.Arguments(1) & " does not exist"
        Wscript.Quit(0)
    End If
    sSrcPath = g_oFSO.GetAbsolutePathName(g_oFSO.GetParentFolderName(sSrcPath))
    sDstPath = g_oFSO.GetAbsolutePathname(g_oFSO.GetParentFolderName(sDstPath))

    ' Build the src and dst dictionaries
    Set oSrcDict = BuildDictionary(sSrcPath)
    Set oDstDict = BuildDictionary(sDstPath)

    ' Locate all orphaned source files and find moved file in destination
    Set oSrcRoot = g_oFSO.GetFolder(sSrcPath)
    Set oDstRoot = g_oFSO.GetFolder(sDstPath)
    nCount = FindMovedFiles(oSrcRoot, oDstRoot, oSrcDict, oDstDict, oSrcRoot)
    
    ' Return value is passed to Wscript.Quit as script exit code
    Main = nCount
End Function

'////////////////////////////////////////////////////////////////////////////
' BuildDictionary
' Build a dictionary object containing signatures for all files in tree
'
' sPath             Path of folder at root of tree
' Returns           Created dictionary object
'
Function BuildDictionary(sPath)
    Trace 1, "+++BuildDictionary: " & sPath
    Dim oDict, oFolder
    Set oDict = CreateObject("Scripting.Dictionary")
    oDict.CompareMode = vbTextCompare           ' Ignore case
    Set oFolder = g_oFSO.GetFolder(sPath)
    AddFiles oDict, oFolder                     ' Recursively add files
    Set BuildDictionary = oDict
End Function

Sub AddFiles(ByVal oDict, ByVal oFolder)
    Trace 2, "+++AddFiles: " & oFolder.Path
    Dim oFile, sSig, oSubFolder

    ' Add each file in this folder
    For Each oFile In oFolder.Files             ' For each file..
        sSig = GetSignature(oFile)              ' Get signature
        If oDict.Exists(sSig) Then              ' If it exists..
            oDict.Item(sSig) = oDict.Item(sSig) & ";" & oFile.Path
        Else
            oDict.Add sSig, oFile.Path          ' Add new item
        End If
    Next

    ' Add subfolder files recursively
    For Each oSubFolder in oFolder.SubFolders   ' For each subfolder..
        AddFiles oDict, oSubFolder              ' ..add more files 
    Next
End Sub

'////////////////////////////////////////////////////////////////////////////
' FindMovedFiles
' Finds all moved files by comparing src to dst
'
' oSrcRoot          Source root folder
' oDstRoot          Destination root folder
' oSrcDict          Source file signature dictionary
' oDstDict          Destination file signature dictionary
' oFolder           Current folder to scan (recursively)
' Returns           Count of moved files located
'
Function FindMovedFiles(oSrcRoot, oDstRoot, oSrcDict, oDstDict, oFolder)
    Trace 2, "+++FindMovedFiles: " & oFolder.Path
    Dim nCount, oFile, oSubFolder, sSig, sCmd, sCmd1, sCmd2
    Dim sSrcPath, sDstPath, sSrcRelPath, sDstRelPath
    nCount = 0

    ' Process each file in this folder
    For Each oFile In oFolder.Files
        sSig = GetSignature(oFile)              ' Get file signature
        ' Signature must exist in src and dst trees
        If oSrcDict.Exists(sSig) And oDstDict.Exists(sSig) Then
            ' There must be only one src and dst file
            If Instr(oSrcDict.Item(sSig), ";") = 0 Then
                If Instr(oDstDict.Item(sSig), ";") = 0 Then
                    ' Src/dst relative paths must be different
                    sSrcPath = oSrcDict.Item(sSig)
                    sDstPath = oDstDict.Item(sSig)
                    sSrcRelPath = Mid(sSrcPath, Len(oSrcRoot.Path) + 1)
                    sDstRelPath = Mid(sDstPath, Len(oDstRoot.Path) + 1)
                    If StrComp(sSrcRelPath, sDstRelPath, vbTextCompare) <> 0 Then
                        sCmd1 = g_oFSO.BuildPath(oDstRoot.Path, sDstRelPath)
                        sCmd2 = g_oFSO.GetParentFolderName(g_oFSO.BuildPath(oDstRoot.Path, sSrcRelPath))
                        sCmd = "MOVE " & sCmd1 & " " & sCmd2
                        Wscript.StdOut.WriteLine sCmd
                        ' Uncomment the next line to perform the actual MOVE operations
                        'g_oFSO.MoveFile sCmd1, sCmd2
                    End If                  
                End If
            End If
        End If
    Next

    ' Process subfolders recursively
    For Each oSubFolder In oFolder.SubFolders
        nCount = nCount + FindMovedFiles(oSrcRoot, oDstRoot, oSrcDict, oDstDict, oSubFolder)
    Next

    FindMovedFiles = nCount                     ' Return move count
End Function

'////////////////////////////////////////////////////////////////////////////
' GetSignature
' Compute signature for a file object
'
' oFile             File object
' Returns           Signature string for file
'
Function GetSignature(oFile)
    Trace 3, "+++GetSignature: " & oFile.Name
    Dim nAttr
    nAttr = oFile.Attributes And Not (32+64+2048)
    GetSignature = oFile.Name & ";" & CStr(oFile.DateCreated)
    GetSignature = GetSignature & ";" & CStr(oFile.DateLastModified)
    GetSignature = GetSignature & ";" & CStr(oFile.Size) & ";" & CStr(nAttr)
End Function

'////////////////////////////////////////////////////////////////////////////
' ShowHelpMessage
' Display help message
'
Sub ShowHelpMessage
    Trace 1, "+++ShowHelpMessage"
    Wscript.Echo "usage: findmovedfiles <src-path> <dst-path>"
    Wscript.Echo "The trees located at src-path and dst-path are compared"
    Wscript.Echo "and files that are at a different location in dst-path"
    Wscript.Echo "(i.e. have been moved) are identified. The output"
    Wscript.Echo "is a list of these files expressed as a series of"
    Wscript.Echo "MOVE commands, which can be captured to a file and"
    Wscript.Echo "executed as a batch script."
End Sub

'////////////////////////////////////////////////////////////////////////////
' Trace
' Debug trace output controlled by g_nTraceLevel and WSHTRACE env var
'
' nLevel            Trace level. Only display if >= WSHTRACE env var
' sText             Text to display
'
Sub Trace(nLevel, sText)
    if g_nTraceLevel > nLevel Then Wscript.Echo sText
End Sub

'////////////////////////////////////////////////////////////////////////////
 