'////////////////////////////////////////////////////////////////////////////
' $Workfile: MTPLogon.vbs $ $Revision: 6 $ $Date: 7/25/99 3:25p $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/MTPLogon.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Phase 2 logon script

' Explicit variable declaration and standard globals
Option Explicit
Dim g_oShell, g_oFSO, g_oNet, g_oGroupDict
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")
Set g_oNet = CreateObject("Wscript.Network")

' A drive map is comprised "<drive>,<UNCshare>"
' Use * in either field for the home drive or directory
Dim g_sDriveMap(25)
g_sDriveMap(0) = "J:,\\K140\JAZDRIVE"
g_sDriveMap(1) = "L:,\\K140\LIBRARY"
g_sDriveMap(2) = "S:,\\K140\SETUP"
g_sDriveMap(3) = "P:,*"
g_sDriveMap(4) = "K:,\\K140\COMMON"
g_sDriveMap(5) = "R:,\\K140\C$"

' Map drives based upon drive mapping table
Dim sMap, bResult, sItems
For Each sMap In g_sDriveMap
    If Not IsEmpty(sMap) Then
        sItems = Split(sMap, ",")
        bResult = MapDrive(sItems(0), sItems(1))
        If Not bResult Then Wscript.Echo "*** FAILED"
    End If
Next

'Map drives based upon group membership
If IsMember("Domain Admins") Then
    bResult = MapDrive("W:", "\\K140\NetApps")
    If Not bResult Then Wscript.Echo "*** FAILED"
End If

Wscript.Quit(0)

'////////////////////////////////////////////////////////////////////////////
' IsMember
' Test to see if user if member of specified group
'
' sGroup            Name of group
' Returns           True if user is group member
'
Function IsMember(sGroup)
    Dim sAdsPath, oUser, oGroup
    ' Populate dictionary if not yet created
    If IsEmpty(g_oGroupDict) Then
        Set g_oGroupDict = CreateObject("Scripting.Dictionary")
        g_oGroupDict.CompareMode = vbTextCompare
        sAdsPath = g_oNet.UserDomain & "/" & g_oNet.UserName
        Set oUser = GetObject("WinNT://" & sAdsPath & ",user")
        For Each oGroup In oUser.Groups
            g_oGroupDict.Add oGroup.Name, "-"
        Next
        Set oUser = Nothing
    End If
    IsMember = CBool(g_oGroupDict.Exists(sGroup))
End Function

'////////////////////////////////////////////////////////////////////////////
' MapDrive
' Map a network drive as specified
'
' g_oFSO            FileSystem object
' g_oNet            WshNetwork object
' sPath             Path name for drive or * (can be relative)
' sUNCPath          UNC path to share or * for home (\\computer\sharename)
' Returns           True if drive mapped ok, else error
'
Function MapDrive(ByVal sPath, ByVal sUNCPath)
    Dim nExit, sAdsPath, oAdsObj, sComputer, sShare, x, sDrive, sLocalPath
    ' If required, get home dir or drive
    If sPath = "*" Or sUNCPath = "*" Then
        sAdsPath = g_oNet.UserDomain & "/" & g_oNet.UserName
        Set oAdsObj = GetObject("WinNT://" & sAdsPath & ",user")
        If sPath = "*" Then sPath = oAdsObj.HomeDirDrive
        If sUNCPath = "*" Then sUNCPath = oAdsObj.HomeDirectory
        Set oAdsObj = Nothing
    End If
    ' Validate the parameters, then parse the UNC name
    If sPath = "" Or sUNCPath = "" Then
        MapDrive = False : Exit Function
    End If
    x = Split(sUNCPath, "\")
    If UBound(x) < 3 Then
        MapDrive = False : Exit Function
    End If
    sComputer = x(2) : sShare = x(3)
    sDrive = GetDriveName(sPath)
    Wscript.Echo "Map: " & sDrive & " -> " & sUNCPath
    ' Try to unmap or unSUBST the drive
    UnmapDrive(sDrive)
    ' Cannot map local drives
    If IsLocalDrive(sDrive) Then
        MapDrive = False : Exit Function
    End If
    ' If possible, use SUBST as this is faster than local loopback
    If IsWinNT And StrComp(sComputer, g_oNet.ComputerName, vbTextCompare) = 0 Then
        sAdsPath = sComputer & "/lanmanserver/" & sShare
        Set oAdsObj = GetObject("WinNT://" & sAdsPath & ",fileshare")
        sLocalPath = oAdsObj.Path
        Set oAdsObj = Nothing
        nExit = RunCmd("subst " & sDrive & " " & sLocalPath)
        ' All done if good SUBST mapping
        If nExit = 0 Then
            MapDrive = True : Exit Function
        End If
    End If
    ' Map the network drive and return result
    g_oNet.MapNetworkDrive sDrive, sUNCPath
    MapDrive = IsNetworkDrive(sDrive)
End Function

'////////////////////////////////////////////////////////////////////////////
' UnmapDrive
' Unmap a network drive (or a SUBST drive on WinNT)
'
' g_oFSO            FileSystem object
' g_oNet            WshNetwork object
' sPath             Path name for drive (can be relative)
'
Sub UnmapDrive(sPath)
    Dim nExit, sDrive
    sDrive = GetDriveName(sPath)
    ' On WinNT, SUBST drives appear as local
    If IsWinNT And IsLocalDrive(sDrive) Then
        nExit = RunCmd("subst " & sDrive & " /d")
    End If
    ' Can only unmap network drives
    If IsNetworkDrive(sDrive) Then
        g_oNet.RemoveNetworkDrive sDrive, True, True
    End If
End Sub

'////////////////////////////////////////////////////////////////////////////
' IsNetworkDrive
' Tests to see if a drive is a networked drive
'
' g_oFSO            FileSystem object
' sPath             Path name for drive (can be relative)
' Returns           True if drive is networked, False otherwise
'
Function IsNetworkDrive(sPath)
    Dim oDrive
    Set oDrive = GetDriveObject(sPath)
    ' Test type if valid drive
    If Not oDrive Is Nothing Then
        IsNetworkDrive = CBool(oDrive.DriveType = 3)
    Else
        IsNetworkDrive = False
    End If
End Function

'////////////////////////////////////////////////////////////////////////////
' IsLocalDrive
' Tests to see if a drive is a local drive (any type)
'
' g_oFSO            FileSystem object
' sPath             Path name for drive (can be relative)
' Returns           True if drive is local, False otherwise
'
Function IsLocalDrive(sPath)
    Dim oDrive
    Set oDrive = GetDriveObject(sPath)
    ' Test type if valid drive
    If Not oDrive Is Nothing Then
        IsLocalDrive = CBool(oDrive.DriveType <> 3)
    Else
        IsLocalDrive = False
    End If
End Function

'////////////////////////////////////////////////////////////////////////////
' GetDriveObject
' Get FSO drive object with error checking
'
' g_oFSO            FileSystem object
' sPath             Path name for drive (can be relative)
' Returns           Drive object or Nothing if drive does not exist
'
Function GetDriveObject(sPath)
    On Error Resume Next
    Set GetDriveObject = Nothing
    Set GetDriveObject = g_oFSO.GetDrive(GetDriveName(sPath))
    If Err.Number <> 0 Then Set GetDriveObject = Nothing
End Function

'////////////////////////////////////////////////////////////////////////////
' GetDriveName
' Get drive name from path name
'
' g_oFSO            FileSystem object
' sPath             Path name for drive (can be relative)
' Returns           Drive name from path
'
Function GetDriveName(sPath)
    GetDriveName = g_oFSO.GetDriveName(g_oFSO.GetAbsolutePathName(sPath))
End Function

'////////////////////////////////////////////////////////////////////////////
' RunCmd
' Run a shell command
'
' g_oShell          WshShell object
' sCmd              Command to execute
' Returns           Exit code from application
'
Function RunCmd(sCmd)
    Dim sShell
    ' Get name of command processor
    sShell = g_oShell.Environment("Process").Item("COMSPEC")
    If sShell = "" Then
        If IsWinNT Then sShell = "cmd" Else sShell = "command"
    End If
    ' Run command via "COMSPEC /c command"
    WScript.Echo "Run: " & sCmd
    RunCmd = g_oShell.Run(sShell & " /c " & sCmd, &H20000000, True)
End Function

'////////////////////////////////////////////////////////////////////////////
' IsWinNT
' Tests for Windows NT or Windows 2000
'
' g_oShell          WshShell object
' Returns           True on NT or Win2K, False otherwise
'
Function IsWinNT
    Dim sOS
    sOS = g_oShell.Environment("Process").Item("OS")
    IsWinNT = CBool(sOS = "Windows_NT")
End Function

'////////////////////////////////////////////////////////////////////////////
