'////////////////////////////////////////////////////////////////////////////
' $Workfile: NewUser.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/NewUser.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Create a new user in the specified computer or domain

' Explicit variable declaration and standard globals
Option Explicit
Dim g_sScriptPath, g_sScriptName, g_sScriptFolder, g_sVersion
Dim g_nTraceLevel
Dim g_oShell, g_oFSO
Dim s, ix, i

' Set standard globals and create global objects
g_sVersion = "1.0"
g_sScriptPath = Wscript.ScriptFullName
g_sScriptName = Wscript.ScriptName
g_sScriptFolder = Left(g_sScriptPath, Len(g_sScriptPath) - Len(g_sScriptName))
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")

' Setup trace control from WSHTRACE environment variable
i = g_oShell.Environment("Process").Item("WSHTRACE")
If IsNumeric(i) Then g_nTraceLevel = CInt(i) Else g_nTraceLevel = 0

' Check for -help, -? etc help request on command line
If Wscript.Arguments.Count > 0 Then
    s = LCase(Wscript.Arguments(0))
    If (s = "-help") Or (s = "-?") Or (s = "/help") Or (s = "/?") Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
End If

' Globals used by Main etc
Dim g_bDomainSwitch, g_sFullNameSwitch, g_sDescriptionSwitch
Dim g_sLoginScriptSwitch, g_sHomeDirSwitch, g_sHomeDriveSwitch
Dim g_aGroupList, g_sHomeComputerSwitch, g_sProtoFilesSwitch
Dim g_sPasswordSwitch, g_sComputerName, g_sUserName

' Show signon banner, then call Main function
ix = Instr(g_sScriptName, ".")
If ix <> 0 Then s = Left(g_sScriptName, ix - 1) Else s = g_sScriptName
Wscript.Echo s & " version " & g_sVersion & vbCRLF
i = Main

' Release standard global objects, then exit script
Set g_oFSO = Nothing
Set g_oShell = Nothing
Wscript.Quit(i)

'////////////////////////////////////////////////////////////////////////////
' Main
' Main function called to embody primary script logic
'
' Returns           Exit code from script
'
Function Main
    Trace 1, "+++Main"
    Dim oArgs, oArg, ix, nCount, sAdsPath, oAdsObj
    Dim oUser, oGroup, sGroup

    ' Get command line and parse into globals
    Set oArgs = CreateObject("MTP.CmdArgs")
    oArgs.FromCollection(Wscript.Arguments)     ' Get command line
    If oArgs.TypedCount(1+4) < 2 Then           ' Must have comp+user
        ShowHelpMessage
        Wscript.Quit(0)
    End If
    g_sComputerName = oArgs.TypedItem(1+4, 0)   ' Get computer name
    g_sUserName = oArgs.TypedItem(1+4, 1)       ' .. and user name
    g_bDomainSwitch = oArgs.TestSwitch("Domain")
    g_sFullNameSwitch = oArgs.GetSwitch("FullName", "")
    g_sDescriptionSwitch = oArgs.GetSwitch("Description", "")
    g_sLoginScriptSwitch = oArgs.GetSwitch("LoginScript", "")
    g_sHomeDirSwitch = oArgs.GetSwitch("HomeDir", "")
    g_sHomeDriveSwitch = oArgs.GetSwitch("HomeDrive", "")
    g_sHomeComputerSwitch = oArgs.GetSwitch("HomeComputer", "")
    g_sProtoFilesSwitch = oArgs.GetSwitch("ProtoFiles", "")
    g_sPasswordSwitch = oArgs.GetSwitch("Password", "")
    ix = 0 : nCount = 0 : ReDim g_aGroupList(100)
    Do
        Set oArg = oArgs.FindArg(2, "Group", ix) ' Get Group arg..
        If Not oArg Is Nothing Then
            ix = oArg.Index + 1                 ' Set search index
            g_aGroupList(nCount) = oArg.Value   ' Get group name
            nCount = nCount + 1                 ' Bump counter
        End If
    Loop Until oArg Is Nothing
    If nCount > 0 Then                          ' If some groups..
        ReDim Preserve g_aGroupList(nCount - 1) ' Trim array
    Else
        g_aGroupList = Empty                    ' Mark no groups
    End If
    Set oArgs = Nothing

    ' Prepare the ADSI object
    sAdsPath = "WinNT://" & g_sComputerName     ' Basic path
    If g_bDomainSwitch Then                     ' If its a domain..
        sAdsPath = sAdsPath & ",domain"
    Else
        sAdsPath = sAdsPath & ",computer"
    End If
    Set oAdsObj = GetObject(sAdsPath)

    ' If /HomeComputer specified, create the share and edit the home dir
    If g_sHomeComputerSwitch <> "" And g_sHomeDirSwitch <> "" Then
        g_sHomeDirSwitch = CreateHomeShare(g_sHomeComputerSwitch, g_sUserName, g_sHomeDirSwitch)
    End If

    ' Create the basic account
    Set oUser = oAdsObj.Create("user", g_sUserName)
    oUser.FullName = g_sFullNameSwitch          ' Set users full name
    oUser.Description = g_sDescriptionSwitch    ' .. and description
    oUser.LoginScript = g_sLoginScriptSwitch    ' Login script name
    oUser.HomeDirectory = g_sHomeDirSwitch      ' Home directory
    oUser.HomeDirDrive = g_sHomeDriveSwitch     ' .. and drive
    oUser.SetInfo                               ' Update the object
    If g_sPasswordSwitch <> "" Then
        oUser.SetPassword = g_sPasswordSwitch   ' Set the password
    End If
    oUser.SetInfo                               ' Update the object

    ' Add the new account to any required groups
    If Not IsEmpty(g_aGroupList) Then           ' If some specified..
        For Each sGroup In g_aGroupList         ' For each group..
            Set oGroup = GetObject(oAdsObj.AdsPath & "/" & sGroup & ",group")
            oGroup.Add oUser.AdsPath            ' Add the user
            Set oGroup = Nothing
        Next
    End If
    Set oUser = Nothing

    ' Copy the prototype tree to the new home directory
    If g_sProtoFilesSwitch <> "" And g_sHomeDirSwitch <> "" Then
        Wscript.Echo "Copy: " & g_sProtoFilesSwitch & " To: " & g_sHomeDirSwitch
        g_oFSO.CopyFolder g_sProtoFilesSwitch, g_sHomeDirSwitch & "\", False
    End If

    ' Return value is passed to Wscript.Quit as script exit code
    Main = 0
End Function

'////////////////////////////////////////////////////////////////////////////
' CreateHomeShare
' Create a new share on the specified computer
'
' sComputer         Target computer (where to create share)
' sUserName         Name of user account for share
' sPath             Path to share on computer (local path)
' Returns           UNC path to the new share
'
Function CreateHomeShare(sComputer, sUserName, sPath)
    Trace 1, "+++ShowHelpMessage"
    Dim oAdsObj, sAdsPath, oShare

    sAdsPath = "WinNT://" & sComputer & "/lanmanserver,fileservice"
    Set oAdsObj = GetObject(sAdsPath)           ' Get service path
    Set oShare = oAdsObj.Create("fileshare", sUserName & "$")   ' Create share
    oShare.Path = sPath                         ' Set local path of share
    oShare.SetInfo                              ' Save the new share
    Set oShare = Nothing
    Set oAdsObj = Nothing
    CreateHomeShare = "\\" & sComputer & "\" & sUserName & "$"
End Function

'////////////////////////////////////////////////////////////////////////////
' ShowHelpMessage
' Display help message
'
Sub ShowHelpMessage
    Trace 1, "+++ShowHelpMessage"
    Wscript.Echo "usage: newuser <computer> <username> [<switches>]"
    Wscript.Echo
    Wscript.Echo "Creates a new user account on the computer or domain"
    Wscript.Echo "specified as the first argument. The following switches"
    Wscript.Echo "are supported:"
    Wscript.Echo "    /Domain            Creates the account in a domain"
    Wscript.Echo "    /FullName:name     Users full name"
    Wscript.Echo "    /Description:text  Descriptive text for account"
    Wscript.Echo "    /LoginScript:name  Login script name"
    Wscript.Echo "    /HomeDir:path      Path to home directory"
    Wscript.Echo "    /HomeDrive:name    Drive to map to home directory"
    Wscript.Echo "    /Group:name        Make user a member of group"
    Wscript.Echo "    /HomeComputer:name Computer for username$ share"
    Wscript.Echo "    /ProtoFiles:path   Source of prototype files"
    Wscript.Echo "    /Password:pwd      Initial password"
    Wscript.Echo "Multiple /Group switches are allowed. If the"
    Wscript.Echo "/HomeComputer is specified, a new share named username$"
    Wscript.Echo "will be created on the computer at the location in"
    Wscript.Echo "/HomeDir. The actual home dir for the account will then"
    Wscript.Echo "reference this share."
End Sub

'////////////////////////////////////////////////////////////////////////////
' Trace
' Debug trace output controlled by g_nTraceLevel and WSHTRACE env var
'
' nLevel            Trace level. Only display if >= WSHTRACE env var
' sText             Text to display
'
Sub Trace(nLevel, sText)
    if g_nTraceLevel > nLevel Then Wscript.Echo sText
End Sub

'////////////////////////////////////////////////////////////////////////////
 