@echo OFF
@if not "%ECHO%"=="" echo %ECHO%
@if not "%OS%"=="Windows_NT" goto DOSEXIT
rem $Workfile: Oldanimal.bat $ $Revision: 1 $ $Date: 7/18/99 10:42p $
rem $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/Oldanimal.bat $

rem Set local scope and call MAIN procedure
setlocal & pushd & set RET=
	set SCRIPTNAME=%~n0
	set SCRIPTPATH=%~f0
	set DATAFILENAME=%~dpn0.DAT
	if "%DEBUG%"=="1" (set TRACE=echo) else (set TRACE=rem)
	call _mtplib :INIT %SCRIPTPATH%
	if /i {%1}=={/help} (call :HELP %2) & (goto :HELPEXIT)
	if /i {%1}=={/?} (call :HELP %2) & (goto :HELPEXIT)
	call :MAIN %*
	:HELPEXIT
popd & endlocal & set RET=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem HELP procedure
rem Display brief on-line help message
rem
:HELP
if defined TRACE %TRACE% [proc %0 %*]
	echo Syntax: ANIMAL
	echo Plays the ancient and well-known animal computer game.
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem MAIN procedure
rem
rem Arguments:	DATAFILENAME=name of data file for dynamic animal database
rem
:MAIN
if defined TRACE %TRACE% [proc %0 %*]
	rem Load the tree from the datafile
	echo One moment please...
	call :LOADDATAFILE "%DATAFILENAME%"
	set /a NEWANIMALCOUNT=0

	rem Main game loop
	:MAINLOOP
		set /a IX=1
		%COMSPEC% /c choice /n Are you thinking of an animal?
		if errorlevel 2 goto :MAINLOOPEND

		rem Work down the tree
		:TREELOOP
			call :GETANIMALNODE %IX%
			if %NODE_YES% EQU 0 goto :TREELOOPEND
			%COMSPEC% /c choice /n %NODE_TEXT%
			if errorlevel 1 set /a IX=NODE_YES
			if errorlevel 2 set /a IX=NODE_NO
		goto :TREELOOP
		:TREELOOPEND

		rem We're at a leaf: the end-game
		%COMSPEC% /c choice /n Is it a %NODE_TEXT%?
		if errorlevel 2 (
			(call :ADDNEWANIMAL)
		) else (
			(echo I guessed it!)
		)
	goto :MAINLOOP
	:MAINLOOPEND

	rem Game wrap-up
	if %NEWANIMALCOUNT% GTR 0 (
		(echo One moment please...)
		(call :SAVEDATAFILE "%DATAFILENAME%")
		if %NEWANIMALCOUNT% EQU 1 (
			(echo I learned about a new animal!)
		) else (
			(echo I learned about %NEWANIMALCOUNT% new animals!)
		)
	)
	echo Thanks for playing!
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem ADDNEWANIMAL procedure
rem Adds a new animal to the tree
rem
rem Arguments:	NODE=var loaded with current node
rem		IX=current node index
rem
:ADDNEWANIMAL
if defined TRACE %TRACE% [proc %0 %*]
	rem Get the new animal name
	echo I give up! What is the animal you are thinking of?
	:GETANIMALLOOP
	echo (type the name and Ctrl+Z)
	call _mtplib :GETINPUTLINE
	set NEWANIMAL=%RET%
	if "%NEWANIMAL%"=="" (
		(echo Please type an animal name)
		(goto :GETANIMALLOOP)
	)

	rem Get the distinguishing question
	echo Type a yes/no question to distinguish a %NODE_TEXT% from a %NEWANIMAL%:
	:GETQUESTIONLOOP
	echo (type the question and Ctrl+Z)
	call _mtplib :GETINPUTLINE
	set NEWQUESTION=%RET%
	if "%NEWQUESTION%"=="" (
		(echo Please type a question)
		(goto :GETQUESTIONLOOP)
	)
	if "%NEWQUESTION%"=="%NEWQUESTION:?=%" (
		(echo Please type a question ^(with a question mark^))
		(goto :GETQUESTIONLOOP)
	)

	rem Get the answer for the NEW animal
	%COMSPEC% /c choice /n For a %NEWANIMAL%, the answer to this question would be?
	set NEWANSWER=%ERRORLEVEL%

	rem Allocate the new nodes
	set /a NEWNODE_YES=ANIMALCOUNT+1
	set /a NEWNODE_NO=ANIMALCOUNT+2
	set /a ANIMALCOUNT+=2
	set /a NEWANIMALCOUNT+=1

	rem Fill existing node with the new question
	set /a ANIMAL_YES_%IX%_=NEWNODE_YES
	set /a ANIMAL_NO_%IX%_=NEWNODE_NO
	set ANIMAL_TEXT_%IX%_=%NEWQUESTION%

	rem Swap node indexes if new answer is NO
	if "%NEWANSWER%"=="2" (
		set /a T1=NEWNODE_YES
		set /a NEWNODE_YES=NEWNODE_NO
		set /a NEWNODE_NO=T1
	)

	rem Install the new animal in the YES node
	set /a ANIMAL_YES_%NEWNODE_YES%_=0
	set /a ANIMAL_NO_%NEWNODE_YES%_=0
	set ANIMAL_TEXT_%NEWNODE_YES%_=%NEWANIMAL%

	rem Install the old animal in the NO node
	set /a ANIMAL_YES_%NEWNODE_NO%_=0
	set /a ANIMAL_NO_%NEWNODE_NO%_=0
	set ANIMAL_TEXT_%NEWNODE_NO%_=%NODE_TEXT%

	echo Thank you!
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem LOADDATAFILE procedure
rem Load animal data from specified file
rem
rem Arguments:	%1=filename to load
rem
:LOADDATAFILE
if defined TRACE %TRACE% [proc %0 %*]
	set /a ANIMALCOUNT=0
	if exist %1 (
		(for /f "eol=; tokens=1,2* delims=," %%I in ('type %1') do call :LOADDATARECORD %%I %%J "%%K")
	) else (
		(call :LOADDATARECORD 2 3 "Does it have a beak?")
		(call :LOADDATARECORD 0 0 "duck")
		(call :LOADDATARECORD 0 0 "dog")
	)
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem LOADDATARECORD procedure
rem Load animal data from
rem
rem Arguments:	%1=yes answer index (or 0)
rem		%2=no answer index (or 0)
rem		%3=question or animal name (in double quotes)
rem
:LOADDATARECORD
if defined TRACE %TRACE% [proc %0 %*]
	set T1=%3
	set /a ANIMALCOUNT+=1
	set /a ANIMAL_YES_%ANIMALCOUNT%_=%1
	set /a ANIMAL_NO_%ANIMALCOUNT%_=%2
	set ANIMAL_TEXT_%ANIMALCOUNT%_=%T1:"=%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem SAVEDATAFILE procedure
rem Save animal data to specified file
rem
rem Arguments:	%1=filename
rem
:SAVEDATAFILE
if defined TRACE %TRACE% [proc %0 %*]
	echo ;ANIMAL data (%ANIMALCOUNT%) >%1
	for /l %%I in (1,1,%ANIMALCOUNT%) do call :SAVEDATARECORD %1 %%I
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem SAVEDATARECORD procedure
rem Save specified record in file
rem
rem Arguments:	%1=file name to save
rem		%2=index into array
rem
:SAVEDATARECORD
if defined TRACE %TRACE% [proc %0 %*]
	call :GETANIMALNODE %2
	echo %NODE_YES%,%NODE_NO%,%NODE_TEXT%>>%1
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem GETANIMALNODE procedure
rem Get a node into the working node
rem
rem Arguments:	%1=index into animal array
rem
rem Returns:	NODE_...=working node loaded
rem
:GETANIMALNODE
if defined TRACE %TRACE% [proc %0 %*]
	set /a NODE_YES=ANIMAL_YES_%1_
	set /a NODE_NO=ANIMAL_NO_%1_
	set RET=%%ANIMAL_TEXT_%1_%%
	call _mtplib :RESOLVE
	set NODE_TEXT=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem Additional procedures go here...

rem These must be the FINAL LINES in the script...
:DOSEXIT
echo This script requires Windows NT

rem /////////////////////////////////////////////////////////////////////////

