'////////////////////////////////////////////////////////////////////////////
' $Workfile: RDir.vbs $ $Revision: 2 $ $Date: 7/07/99 9:28p $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/RDir.vbs $
' Copyright (c) 1999 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Regulr expression file search using FileScan class

Option Explicit
Dim oFileScan, oFSO, ix
Set oFSO = CreateObject("Scripting.FileSystemObject")

' Show usage if no arguments
If Wscript.Arguments.Count < 1 Then
    Wscript.Echo "usage: filescan reg-expr [path...]"
    Wscript.Quit(0)
End If

' Create the FileScan object and set the RE from cmd line
Set oFileScan = New FileScan
oFileScan.Expr = Wscript.Arguments(0)
oFileScan.ScanSubFolders = True

' Now process all arguments, or use current dir if none
If Wscript.Arguments.Count > 1 Then
    For ix = 1 To Wscript.Arguments.Count - 1
        oFileScan.Execute oFSO.GetAbsolutePathName(Wscript.Arguments(ix))
    Next
Else
    oFileScan.Execute oFSO.GetAbsolutePathName(".")
End If

' Display the results
For ix = 0 To oFileScan.Count
    Wscript.Echo oFileScan(ix)
Next

Wscript.Quit(oFileScan.Count)

'////////////////////////////////////////////////////////////////////////////
' FileScan
' File scan class, builds a collection of files based upon a regular expr
'
Class FileScan
    Public Expr                                 ' Reg expr to match
    Public MatchFullPath                        ' True to match full path
    Public ScanSubFolders                       ' True to scan subfolders
    Private sFiles                              ' List of matching files
    Private nCount                              ' Count of matches

    ' Reset object (delete all file matches)
    Public Sub Reset
        nCount = 0                              ' Mark no files
        sFiles = Empty                          ' Empty the files array
    End Sub

    ' Return current file count (R/O)
    Public Property Get Count
        Count = CInt(nCount)                    ' Just return count
    End Property

    ' Execute search (build file list)
    Public Sub Execute(sPath)
        Dim oFSO, oFolder, oRE
        Set oFSO = CreateObject("Scripting.FileSystemObject")
        Set oFolder = oFSO.GetFolder(sPath)     ' Get folder path
        Set oRE = New RegExp
        oRE.Pattern = Expr                      ' Set match pattern
        oRE.IgnoreCase = True                   ' Ignore case
        MatchFiles oRE, oFolder                 ' Go match files here
    End Sub

    Private Sub MatchFiles(oRE, oFolder)        ' Match file in folder
        Dim oFile, oSubFolder
        For Each oFile In oFolder.Files         ' Scan all files
            If MatchFullPath Then
                If oRE.Test(oFile.Path) Then    ' Full path match
                    AddFile oFile.Path          ' Add if match
                End If
            Else
                If oRE.Test(oFile.Name) Then    ' Name match
                    AddFile oFile.Path          ' Add if match
                End If
            End If
        Next
        If ScanSubFolders Then                  ' If required..
            For Each oSubFolder In oFolder.SubFolders   ' Scan subs
                MatchFiles oRE, oSubFolder
            Next
        End If
    End Sub

    ' Get individual files (default property)
    Public Default Property Get Item(nIndex)    ' Get indexed item
        Dim ixMajor, ixMinor
        If (nIndex >= nCount) Or (nIndex >= 1024 * 4096) Then
            Item = ""                           ' Flag overflow
        Else
            ixMajor = nIndex / 4096             ' Compute major index
            ixMinor = nIndex And 4095           ' Compute minor index
            Item = sFiles(ixMajor)(ixMinor)     ' Get path name
        End If
    End Property

    ' Add new file item to dynamic array
    Private Sub AddFile(sPath)
        Dim ix, ixMajor, ixMinor, a             ' Array indices
        If IsEmpty(sFiles) Then                 ' If no array yet..
            ReDim sFiles(1023)                  ' Space for 1024 sets
        End If
        ix = nCount                             ' Get current index
        nCount = nCount + 1                     ' Bump counter
        If ix >= 1024 * 4096 Then Exit Sub      ' Exit if overflow!!
        ixMajor = ix / 4096                     ' Compute major index
        ixMinor = ix And 4095                   ' Compute minor index
        If IsEmpty(sFiles(ixMajor)) Then        ' If new array needed
            ReDim a(4095)                       ' Create temp array
            sFiles(ixMajor) = a                 ' Copy to populate
        End If
        sFiles(ixMajor)(ixMinor) = sPath        ' Add path to array
    End Sub
End Class
