'////////////////////////////////////////////////////////////////////////////
' $Workfile: VBSToWS.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/VBSToWS.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Converts a .VBS script into a .WS script

' Explicit variable declaration and standard globals
Option Explicit
Dim g_sScriptPath, g_sScriptName, g_sScriptFolder, g_sVersion
Dim g_nTraceLevel
Dim g_oShell, g_oFSO
Dim s, ix, i

' Set standard globals and create global objects
g_sVersion = "1.0"
g_sScriptPath = Wscript.ScriptFullName
g_sScriptName = Wscript.ScriptName
g_sScriptFolder = Left(g_sScriptPath, Len(g_sScriptPath) - Len(g_sScriptName))
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")

' Setup trace control from WSHTRACE environment variable
i = g_oShell.Environment("Process").Item("WSHTRACE")
If IsNumeric(i) Then g_nTraceLevel = CInt(i) Else g_nTraceLevel = 0

' Check for -help, -? etc help request on command line
If Wscript.Arguments.Count > 0 Then
    s = LCase(Wscript.Arguments(0))
    If (s = "-help") Or (s = "-?") Or (s = "/help") Or (s = "/?") Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
End If

' Show signon banner, then call Main function
ix = Instr(g_sScriptName, ".")
If ix <> 0 Then s = Left(g_sScriptName, ix - 1) Else s = g_sScriptName
Wscript.Echo s & " version " & g_sVersion & vbCRLF
i = Main

' Release standard global objects, then exit script
Set g_oFSO = Nothing
Set g_oShell = Nothing
Wscript.Quit(i)

'////////////////////////////////////////////////////////////////////////////
' Main
' Main function called to embody primary script logic
'
' Returns           Exit code from script
'
Function Main
    Trace 1, "+++Main"
    Dim sDstPath, oDstStream, sSrcPath, oSrcStream, ix, sLine, sID

    ' The LAST argument is the .WS output file, others are .VBS i/p files
    If Wscript.Arguments.Count < 2 Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
    sDstPath = Wscript.Arguments(Wscript.Arguments.Count - 1)
    If sDstPath <> "-" Then
        sDstPath = g_oFSO.GetAbsolutePathName(sDstPath)
        Set oDstStream = g_oFSO.CreateTextFile(sDstPath, True)
    Else
        Set oDstStream = Wscript.StdOut
    End If

    ' Emit the preamble
    oDstStream.WriteLine "<?xml verion=""1.0""?>"   ' Special XML header
    oDstStream.WriteLine "<!-- Created by VBStoWS Script -->"
    oDstStream.WriteLine
    If Wscript.Arguments.Count > 2 Then         ' If multiple .VBS files
        oDstStream.WriteLine "<package>"
        oDstStream.WriteLine
    End If

    ' Process each individual .VBS file
    For ix = 0 To Wscript.Arguments.Count - 2
        sSrcPath = g_oFSO.GetAbsolutePathName(Wscript.Arguments(ix))
        sID = g_oFSO.GetBaseName(sSrcPath)

        ' Emit the file preamble
        oDstStream.WriteLine "<!-- " & sSrcPath & " -->"
        oDstStream.WriteLine "<job id=" & MakeQXML(sID) & ">"
        oDstStream.WriteLine "<script language=""VBScript"">"
        oDstStream.WriteLine "<![CDATA["

        ' Copy the .VBS file to the .WS file
        Set oSrcStream = g_oFSO.OpenTextFile(sSrcPath)
        Do Until oSrcStream.AtEndOfStream
            sLine = oSrcStream.ReadLine
            oDstStream.WriteLine sLine
        Loop
        oSrcStream.Close
        Set oSrcStream = Nothing

        ' Emit the file postamble
        oDstStream.WriteLine "]]>"
        oDstStream.WriteLine "</script>"
        oDstStream.WriteLine "</job>"
        oDstStream.WriteLine
    Next

    ' Emit the postamble and close the dest stream
    If Wscript.Arguments.Count > 2 Then         ' If multiple .VBS files
        oDstStream.WriteLine "</package>"
    End If
    oDstStream.Close
    Set oDstStream = Nothing
       
    ' Return value is passed to Wscript.Quit as script exit code
    Main = 0
End Function

'////////////////////////////////////////////////////////////////////////////
' MakeXML and MakeQXML
' Make a string valid for XML use (converts reserved chars to entities)
'
' sString           String to convert
'
Function MakeXML(sString)
    ' Replace reserved characters with entities
    MakeXML = sString                           ' Get replacement string
    MakeXML = Replace(MakeXML, "&", "&amp;")    ' MUST BE FIRST!!
    MakeXML = Replace(MakeXML, "<", "&lt;")     ' Less than
    MakeXML = Replace(MakeXML, ">", "&gt;")     ' Greater than
    MakeXML = Replace(MakeXML, "'", "&apos;")   ' Apostrophe
    MakeXML = Replace(MakeXML, Chr(34), "&quot;")   ' Double quote
End Function

Function MakeQXML(sString)
    MakeQXML = Chr(34) & MakeXML(sString) & Chr(34)
End Function

'////////////////////////////////////////////////////////////////////////////
' ShowHelpMessage
' Display help message
'
Sub ShowHelpMessage
    Trace 1, "+++ShowHelpMessage"
    Wscript.Echo "usage: vbstows <vbs-file> ... <ws-file>"
    Wscript.Echo
    Wscript.Echo "Converts one or more WSH 1.0 style .VBS scripts into a"
    Wscript.Echo "single WSH 2.0 .WS script by adding the necessary XML"
    Wscript.Echo "elements. The last arguments specifies the <ws-file> to"
    Wscript.Echo "use for output. Each .VBS file is written to the .WS file as"
    Wscript.Echo "a single job element. If the <ws-file> is ""-"", the"
    Wscript.Echo "output is written to StdOut."
End Sub

'////////////////////////////////////////////////////////////////////////////
' Trace
' Debug trace output controlled by g_nTraceLevel and WSHTRACE env var
'
' nLevel            Trace level. Only display if >= WSHTRACE env var
' sText             Text to display
'
Sub Trace(nLevel, sText)
    if g_nTraceLevel > nLevel Then Wscript.Echo sText
End Sub

'////////////////////////////////////////////////////////////////////////////
 