'////////////////////////////////////////////////////////////////////////////
' $Workfile: XMLDir.vbs $ $Revision: 2 $ $Date: 7/19/99 12:19a $
' $Archive: /TimH/Projects (SCC)/Books & Articles/Macmillan/Windows 2000 System Scripting (1999)/Scripts/XMLDir.vbs $
' Copyright (c) 1998 Tim Hill. All Rights Reserved.
'////////////////////////////////////////////////////////////////////////////

' Generate an XML representation of a directory

' Explicit variable declaration and standard globals
Option Explicit
Dim g_sScriptPath, g_sScriptName, g_sScriptFolder, g_sVersion
Dim g_nTraceLevel
Dim g_oShell, g_oFSO
Dim s, ix, i

' Set standard globals and create global objects
g_sVersion = "1.0"
g_sScriptPath = Wscript.ScriptFullName
g_sScriptName = Wscript.ScriptName
g_sScriptFolder = Left(g_sScriptPath, Len(g_sScriptPath) - Len(g_sScriptName))
Set g_oShell = CreateObject("Wscript.Shell")
Set g_oFSO = CreateObject("Scripting.FileSystemObject")

' Setup trace control from WSHTRACE environment variable
i = g_oShell.Environment("Process").Item("WSHTRACE")
If IsNumeric(i) Then g_nTraceLevel = CInt(i) Else g_nTraceLevel = 0

' Check for -help, -? etc help request on command line
If Wscript.Arguments.Count > 0 Then
    s = LCase(Wscript.Arguments(0))
    If (s = "-help") Or (s = "-?") Or (s = "/help") Or (s = "/?") Then
        ShowHelpMessage
        Wscript.Quit(1)
    End If
End If

' Show signon banner, then call Main function
ix = Instr(g_sScriptName, ".")
If ix <> 0 Then s = Left(g_sScriptName, ix - 1) Else s = g_sScriptName
Wscript.Echo s & " version " & g_sVersion & vbCRLF
i = Main

' Release standard global objects, then exit script
Set g_oFSO = Nothing
Set g_oShell = Nothing
Wscript.Quit(i)

'////////////////////////////////////////////////////////////////////////////
' Main
' Main function called to embody primary script logic
'
' Returns           Exit code from script
'
Function Main
    Trace 1, "+++Main"
    Dim sPath, oFolder, oFile, oSubFolder

    ' Get the absolute path name of the directory
    If Wscript.Arguments.Count < 1 Then
        sPath = "."                             ' Use this directory
    Else
        sPath = Wscript.Arguments(0)
    End If
    sPath = g_oFSO.GetAbsolutePathName(sPath)   ' Make absolute
    Set oFolder = g_oFSO.GetFolder(sPath)       ' Get the folder

    ' Emit the preamble
    Wscript.StdOut.WriteLine "<?xml verion=""1.0""?>"       ' Special XML header
    Wscript.StdOut.WriteLine "<!-- XML Directory of " & sPath & " -->"
    Wscript.StdOut.WriteLine
    Wscript.StdOut.WriteLine "<DIRECTORY path=" & MakeQXML(sPath) & ">"

    ' Emit individual file and folder elements
    For Each oFile In oFolder.Files             ' For each file..
        EmitFile "    ", oFile                  ' Emit the XML
    Next
    For Each oSubFolder In oFolder.SubFolders   ' For each folder
        EmitFolder "    ", oSubFolder           ' Emit the XML
    Next

    ' Emit the postamble
    Wscript.StdOut.WriteLine "</DIRECTORY>"
    Set oFolder = Nothing
       
    ' Return value is passed to Wscript.Quit as script exit code
    Main = 0
End Function

'////////////////////////////////////////////////////////////////////////////
' EmitFile
' Emit file object as XML
'
Sub EmitFile(sPrefix, oFile)
    Wscript.StdOut.WriteLine sPrefix & "<FILE>"             ' Start element
    EmitElement sPrefix & "    ", "NAME", oFile.Name
    EmitElement sPrefix & "    ", "DATECREATED", oFile.DateCreated
    EmitElement sPrefix & "    ", "DATELASTACCESSED", oFile.DateLastAccessed
    EmitElement sPrefix & "    ", "DATELASTMODIFIED", oFile.DateLastModified
    EmitElement sPrefix & "    ", "SIZE", oFile.Size
    EmitElement sPrefix & "    ", "ATTR", GetAttrString(oFile.Attributes)
    Wscript.StdOut.WriteLine sPrefix & "</FILE>"            ' Close element
End Sub

'////////////////////////////////////////////////////////////////////////////
' EmitFolder
' Emit folder object as XML
'
Sub EmitFolder(sPrefix, oFolder)
    Wscript.StdOut.WriteLine sPrefix & "<FOLDER>"               ' Start element
    EmitElement sPrefix & "    ", "NAME", oFolder.Name
    EmitElement sPrefix & "    ", "DATECREATED", oFolder.DateCreated
    EmitElement sPrefix & "    ", "DATELASTACCESSED", oFolder.DateLastAccessed
    EmitElement sPrefix & "    ", "DATELASTMODIFIED", oFolder.DateLastModified
    EmitElement sPrefix & "    ", "ATTR", GetAttrString(oFolder.Attributes)
    Wscript.StdOut.WriteLine sPrefix & "</FOLDER>"          ' Close element
End Sub

'////////////////////////////////////////////////////////////////////////////
' GetAttrString
' Get string form of attributes
'
Function GetAttrString(nAttr)
    Dim sAttr
    GetAttrString = ""
    If nAttr And 1 Then sAttr = "R" Else sAttr = "-"
    GetAttrString = GetAttrString & sAttr
    If nAttr And 2 Then sAttr = "H" Else sAttr = "-"
    GetAttrString = GetAttrString & sAttr
    If nAttr And 4 Then sAttr = "S" Else sAttr = "-"
    GetAttrString = GetAttrString & sAttr
    If nAttr And 8 Then sAttr = "V" Else sAttr = "-"
    GetAttrString = GetAttrString & sAttr
    If nAttr And 16 Then sAttr = "D" Else sAttr = "-"
    GetAttrString = GetAttrString & sAttr
    If nAttr And 32 Then sAttr = "A" Else sAttr = "-"
    GetAttrString = GetAttrString & sAttr
End Function

'////////////////////////////////////////////////////////////////////////////
' EmitElement
' Emit individual XML element
'
Sub EmitElement(sPrefix, sName, sValue)
    Dim sElement

    sElement = "<" & sName & ">" & MakeXML(sValue) & "</" & sName & ">"
    Wscript.StdOut.WriteLine sPrefix & sElement
End Sub

'////////////////////////////////////////////////////////////////////////////
' MakeXML and MakeQXML
' Make a string valid for XML use (converts reserved chars to entities)
'
' sString           String to convert
'
Function MakeXML(sString)
    ' Replace reserved characters with entities
    MakeXML = sString                           ' Get replacement string
    MakeXML = Replace(MakeXML, "&", "&amp;")    ' MUST BE FIRST!!
    MakeXML = Replace(MakeXML, "<", "&lt;")     ' Less than
    MakeXML = Replace(MakeXML, ">", "&gt;")     ' Greater than
    MakeXML = Replace(MakeXML, "'", "&apos;")   ' Apostrophe
    MakeXML = Replace(MakeXML, Chr(34), "&quot;")   ' Double quote
End Function

Function MakeQXML(sString)
    MakeQXML = Chr(34) & MakeXML(sString) & Chr(34)
End Function

'////////////////////////////////////////////////////////////////////////////
' ShowHelpMessage
' Display help message
'
Sub ShowHelpMessage
    Trace 1, "+++ShowHelpMessage"
    Wscript.Echo "usage: xmldir [<path>]"
    Wscript.Echo
    Wscript.Echo "Generates an XML file to StdOut containing a well-formed"
    Wscript.Echo "XML file (w/o a DTD) the describes the contents of the"
    Wscript.Echo "specified directory, or the current directory if none is"
    Wscript.Echo "specified."
End Sub

'////////////////////////////////////////////////////////////////////////////
' Trace
' Debug trace output controlled by g_nTraceLevel and WSHTRACE env var
'
' nLevel            Trace level. Only display if >= WSHTRACE env var
' sText             Text to display
'
Sub Trace(nLevel, sText)
    if g_nTraceLevel > nLevel Then Wscript.Echo sText
End Sub

'////////////////////////////////////////////////////////////////////////////
 